package com.zd.airbottle.mapper;

import java.util.List;
import com.zd.airbottle.domain.QpTransportCarCredentials;
import org.apache.ibatis.annotations.Param;

/**
 * 运输车辆证件Mapper接口
 * 
 * @author zd
 * @date 2022-05-12
 */
public interface QpTransportCarCredentialsMapper 
{
    /**
     * 查询运输车辆证件
     * 
     * @param id 运输车辆证件主键
     * @return 运输车辆证件
     */
    QpTransportCarCredentials selectQpTransportCarCredentialsById(Long id);

    /**
     * 查询运输车辆证件列表
     * 
     * @param qpTransportCarCredentials 运输车辆证件
     * @return 运输车辆证件集合
     */
    List<QpTransportCarCredentials> selectQpTransportCarCredentialsList(QpTransportCarCredentials qpTransportCarCredentials);

    /**
     * 新增运输车辆证件
     * 
     * @param qpTransportCarCredentials 运输车辆证件
     * @return 结果
     */
    int insertQpTransportCarCredentials(QpTransportCarCredentials qpTransportCarCredentials);

    /**
     * 修改运输车辆证件
     * 
     * @param qpTransportCarCredentials 运输车辆证件
     * @return 结果
     */
    int updateQpTransportCarCredentials(QpTransportCarCredentials qpTransportCarCredentials);

    /**
     * 删除运输车辆证件
     * 
     * @param id 运输车辆证件主键
     * @return 结果
     */
    int deleteQpTransportCarCredentialsById(Long id);

    /**
     * 批量删除运输车辆证件
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteQpTransportCarCredentialsByIds(Long[] ids);

    /**
     * 根据车辆ID查询运输车辆证件
     *
     * @param carId 运输车辆主键
     * @return 运输车辆证件
     */
    QpTransportCarCredentials getCarCredentialsByCarId(@Param("carId") Long carId);

    /**
     * 根据车辆ID集合查询运输车辆证件
     * @param carIds 运输车辆主键
     * @return 运输车辆证件集合
     */
    List<QpTransportCarCredentials> getByCarIds(List<Long> carIds);
}
